import matplotlib.pyplot as plt

from . import container

#Inputs: x = stock price, strike = strike price, p = premium
#Output: Net Profit or Loss

def longCall(x, strike, p):
	if x < strike:
		return -p
	elif x >= strike:
		return x - strike - p
	else:
		return None

def shortCall(x, strike, p):
	if x < strike:
		return p
	elif x >= strike:
		return p - (x - strike)
	else:
		return None

def longPut(x, strike, p):
	if x < strike:
		return strike - x - p
	elif x >= strike:
		return -p


def shortPut(x, strike, p):
	if x < strike:
		return p + (x - strike)
	elif x >= strike:
		return p

def graphCalls(optionObj, contractName, display = True, export = False, output = 'graph.pdf'):

	s = float(optionObj.getAttr(contractName, 'Strike'))
	lp = float(optionObj.getAttr(contractName, 'Last Price'))

	x = [i for i in range(round(float(optionObj.stockPrice)) // 2, round(float(optionObj.stockPrice)) * 2)]

	y_long = [longCall(i, s, lp) for i in x]
	y_short = [shortCall(i, s, lp) for i in x]

	fig, axs = plt.subplots(2)
	fig.suptitle("Call Option: " + contractName)
	axs[0].axvline(x = optionObj.stockPrice)
	axs[0].plot(x, y_long)
	axs[0].title.set_text("Long Call")
	axs[0].set_xlabel("Stock Price at Expiration")
	axs[0].set_ylabel("Profit/Loss ($)")

	axs[1].plot(x, y_short)
	axs[1].title.set_text("Short Call")
	axs[1].set_xlabel("Stock Price at Expiration")
	axs[1].set_ylabel("Profit/Loss ($)")
	# axs[1].axvline(x = optionObj.stockPrice)

	fig = plt.gcf().subplots_adjust(hspace = 0.5)
	# fig.subplots_adjust(hspace = 0.2)

	if export == True:
		plt.savefig(output)

	if display == True:
		plt.show()

def graphPuts(optionObj, contractName, display = True, export = False, output = 'graph.png'):

	s = float(optionObj.getAttr(contractName, 'Strike'))
	lp = float(optionObj.getAttr(contractName, 'Last Price'))

	x = [i for i in range(round(float(optionObj.stockPrice)) // 2, round(float(optionObj.stockPrice)) * 2)]

	y_long = [longPut(i, s, lp) for i in x]
	y_short = [shortPut(i, s, lp) for i in x]

	fig, axs = plt.subplots(2)
	fig.suptitle("Put Option: " + contractName)
	axs[0].plot(x, y_long)
	axs[0].title.set_text("Long Put")
	axs[0].set_xlabel("Stock Price at Expiration")
	axs[0].set_ylabel("Profit/Loss ($)")

	axs[1].plot(x, y_short)
	axs[1].title.set_text("Short Put")
	axs[1].set_xlabel("Stock Price at Expiration")
	axs[1].set_ylabel("Profit/Loss ($)")

	fig = plt.gcf().subplots_adjust(hspace = 0.5)
	# fig.subplots_adjust(hspace = 0.2)

	if export == True:
		plt.savefig(output)

	if display == True:
		plt.show()



def graphLongCall(optionObj, contractName, export = False):
	s = float(optionObj.getAttr(contractName, 'Strike'))
	lp = float(optionObj.getAttr(contractName, 'Last Price'))

	x = [i for i in range(round(float(optionObj.stockPrice)) // 2, round(float(optionObj.stockPrice)) * 2)]
	y = [longCall(i, s, lp) for i in x]

	plot_title = contractName
	plt.title(plot_title)
	plt.plot(x, y)

	if export == True:
		plt.savefig('options.png')

	plt.show()

def graphShortCall(optionObj, contractName, export = False):
	s = float(str(optionObj.getAttr(contractName, 'Strike')).replace(',', '').replace('\n', ''))
	lp = float(optionObj.getAttr(contractName, 'Last Price'))

	x = [i for i in range(round(float(optionObj.stockPrice)) // 2, round(float(optionObj.stockPrice)) * 2)]
	y = [shortCall(i, s, lp) for i in x]

	plot_title = contractName
	plt.title(plot_title)
	plt.plot(x, y)

	if export == True:
		plt.savefig('options.png')

	plt.show()

def graphLongPut(optionObj, contractName, export = False):
	s = float(str(optionObj.getAttr(contractName, 'Strike')).replace(',', '').replace('\n', ''))
	lp = float(optionObj.getAttr(contractName, 'Last Price'))

	x = [i for i in range(round(float(optionObj.stockPrice)) // 2, round(float(optionObj.stockPrice)) * 2)]
	y = [longPut(i, s, lp) for i in x]

	plot_title = contractName
	plt.title(plot_title)
	plt.plot(x, y)

	if export == True:
		plt.savefig('options.png')

	plt.show()

def graphShortPut(optionObj, contractName, export = False):
	s = float(str(optionObj.getAttr(contractName, 'Strike')).replace(',', '').replace('\n', ''))
	lp = float(optionObj.getAttr(contractName, 'Last Price'))

	x = [i for i in range(round(float(optionObj.stockPrice)) // 2, round(float(optionObj.stockPrice)) * 2)]
	y = [shortPut(i, s, lp) for i in x]

	plot_title = contractName
	plt.title(plot_title)
	plt.plot(x, y)

	if export == True:
		plt.savefig('options.png')

	plt.show()



#Python modules
import requests
import bs4
import pandas as pd
import numpy

#Local imports
from . import container
from . import exceptions


def scrapePrice(tickerSymbol):
	r = requests.get('https://finance.yahoo.com/quote/' + tickerSymbol)
	soup = bs4.BeautifulSoup(r.text, 'lxml')
	price = soup.find('div', {'class':'D(ib) Mend(20px)'}).find('span').text

	return price


def scrapeCallOptions(tickerSymbol):
	#Load the webpage content
	url = 'https://finance.yahoo.com/quote/' + tickerSymbol + '/options?p=' + tickerSymbol
	r = requests.get(url) 

	#Convert to bs object
	webpage = bs4.BeautifulSoup(r.content, features='lxml')

	date = webpage.select('section > section > div > span', {'datareactid':'47'})[2].text

	#Scrape data
	tables = webpage.select('table', attrs={'datareactid':'50'})

	callTable = tables[0]
	putTable = tables[1]

	columns = callTable.find('thead').findAll('th')
	columnNames = [c.text for c in columns]


	rows = callTable.find('tbody').findAll('tr')
	optionMatrix = []

	for row in rows:
		tds = row.findAll('td')
		option = [td.get_text() for td in tds]
		optionMatrix.append(option)

	try:
		df = pd.DataFrame(optionMatrix, columns=columnNames)
		if len(df.index) == 0:
			raise exceptions.EmptyOptionChainError
	except exceptions.EmptyOptionChainError:
		print("Scraped option chain was empty")

	optionObj = container.OptionObj(df, date, scrapePrice(tickerSymbol))

	return optionObj

def scrapePutOptions(tickerSymbol):
	#load the webpage content
	url = 'https://finance.yahoo.com/quote/' + tickerSymbol + '/options?p=' + tickerSymbol
	r = requests.get(url)

	webpage = bs4.BeautifulSoup(r.content, features='lxml')

	date = webpage.select('section > section > div > span', {'datareactid':'47'})[2].text

	tables = webpage.select("table", attrs={'datareactid':'50'})

	putTable = tables[1]

	columns = putTable.find('thead').findAll('th')
	columnNames = [c.get_text() for c in columns]

	rows = putTable.find('tbody').findAll('tr')
	optionMatrix = []

	for row in rows:
		tds = row.findAll('td')
		option = [td.get_text() for td in tds]
		optionMatrix.append(option)

	try:
		df = pd.DataFrame(optionMatrix, columns=columnNames)
		if len(df.index) == 0:
			raise exceptions.EmptyOptionChainError
	except exceptions.EmptyOptionChainError:
		print("Scraped option chain was empty")

	optionObj = container.OptionObj(df, date, scrapePrice(tickerSymbol))

	return optionObj

def scrapeAggregateOptions(tickerSymbol):
	#load the webpage content
	url = 'https://finance.yahoo.com/quote/' + tickerSymbol + '/options?p=' + tickerSymbol
	r = requests.get(url)

	webpage = bs4.BeautifulSoup(r.content, features='lxml')

	date = webpage.select('section > section > div > span', {'datareactid':'47'})[2].text

	tables = webpage.select("table", attrs={'datareactid':'50'})

	callTable = tables[0]
	putTable = tables[1]

	put_columns = putTable.find('thead').findAll('th')
	put_columnNames = [c.get_text() for c in put_columns]

	put_rows = putTable.find('tbody').findAll('tr')
	put_optionMatrix = []

	for row in put_rows:
		tds = row.findAll('td')
		option = [td.get_text() for td in tds]
		put_optionMatrix.append(option)

	call_columns = callTable.find('thead').findAll('th')
	call_columnNames = [c.get_text() for c in call_columns]

	call_rows = callTable.find('tbody').findAll('tr')
	call_optionMatrix = []

	for row in call_rows:
		tds = row.findAll('td')
		option = [td.get_text() for td in tds]
		call_optionMatrix.append(option)


	try:
		call_df = pd.DataFrame(call_optionMatrix, columns=call_columnNames)
		put_df = pd.DataFrame(put_optionMatrix, columns = put_columnNames)
		if len(call_df.index) == 0 or len(put_df.index) == 0:
			raise exceptions.EmptyOptionChainError
	except exceptions.EmptyOptionChainError:
		print("Scraped option chain was empty")

	optionObj = container.AggregateOptionObj(call_df, put_df, date, scrapePrice(tickerSymbol)) 

	return optionObj




from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="vops",
    version="0.0.23",
    author="Ryan Lee",
    author_email="ryanjlee22@gmail.com",
    description="A package for graphing profit-loss option diagrams",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/RJL22/vops",
    package_dir={"": 'src'},
    packages=find_packages(
    	where="src"
    	),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires= [
    "beautifulsoup4==4.9.3",
	"bs4==0.0.1",
	"certifi==2020.11.8",
	"chardet==3.0.4",
	"cycler==0.10.0",
	"idna==2.10",
	"kiwisolver==1.3.1",
	"lxml==4.6.1",
	"matplotlib==3.3.3",
	"numpy==1.19.4",
	"pandas==1.1.4",
	"Pillow==8.0.1",
	"pyparsing==2.4.7",
	"python-dateutil==2.8.1",
	"pytz==2020.4",
	"requests==2.25.0",
	"six==1.15.0",
	"soupsieve==2.0.1",
	"urllib3==1.26.2",
    ],
    python_requires='>=3.6',
)


class OptionObj:
	def __init__(self):
		self.chain = None
		self.expiration = None
		self.stockPrice = None

	def __init__(self, a_chain, a_expiration, a_stockPrice):
		self.chain = a_chain
		self.expiration = a_expiration
		self.stockPrice = a_stockPrice

	def getExpiration(self):
		return self.expiration

	def getChain(self):
		return self.chain

	def getAttr(self, contractName, attr):
		colNum = self.chain.columns.get_loc(attr)
		row = self.chain.loc[self.chain['Contract Name'] == contractName]

		return row.iloc[0, colNum].replace(',', '')

class AggregateOptionObj:
	def __init__(self):
		self.call_chain = None
		self.put_chain = None
		self.expiration = None
		self.stockPrice = None

	def __init__(self, a_call_chain, a_put_chain, a_expiration, a_stockPrice):
		self.call_chain = a_call_chain
		self.put_chain = a_put_chain
		self.expiration = a_expiration
		self.stockPrice = a_stockPrice

	def getExpiration(self):
		return self.expiration

	def getCallChain(self):
		return self.call_chain

	def getPutChain(self):
		return self.put_chain

	def getAttr(self, contractName, type="Call", attr):
		if type == "Call":
			colNum = self.call_chain.columns.get_loc(attr)
			row = self.call_chain.loc[self.call_chain['Contract Name'] == contractName]

			return row.iloc[0, colNum].replace(',', '')

		elif type == "Put":
			colNum = self.put_chain.columns.get_loc(attr)
			row = self.put_chain.loc[self.put_chain['Contract Name'] == contractName]

			return row.iloc[0, colNum].replace(',', '')

class CumulativeChain:
	def __init__(self):
		self.chain = None
		self.expiration = None
		self.stockPrice = None

class Option:
	def __init__(self):
		self.contract_name = None
		self.type = None
		self.strike = None
		self.last_price = None


